package de.comline.spring.service;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.is;
import  static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;
import static org.springframework.test.annotation.DirtiesContext.ClassMode.AFTER_EACH_TEST_METHOD;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.test.annotation.DirtiesContext;

import de.comline.spring.entity.Movie;
import de.comline.spring.repository.MovieRepository;
import de.comline.spring.service.MovieServiceTestWithMockRepo.TestConfig;

@SpringBootTest(classes = {
	MovieService.class,
	TestConfig.class
})
@DirtiesContext(classMode = AFTER_EACH_TEST_METHOD)
public class MovieServiceTestWithMockRepo {
	@Autowired
	private MovieService service;
	
	@Test
	@DisplayName("Test with a mock movie repository")
	void withMockRepository() {
		String title = "the-movie-title";
		
		Movie movie = service.createMovie(title);
		assertThat(movie.getTitle(), is(title));
	}
	
	@Configuration
	static class TestConfig {
		@Bean
		MovieRepository createMockRepository() {
			MovieRepository mockRepository = mock(MovieRepository.class);
			when(mockRepository.save(any())).thenAnswer(invocation -> {
				return invocation.getArgument(0);
			});
			return mockRepository;
		}
	}
}
